@@ -39,10 +39,10 @@
  *
  * Last reviewed on 2006-03-01 (0.10.4)
  */
-
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
+
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
@@ -51,13 +51,29 @@
 #include <gst/gsttagsetter.h>
 #include <gst/tag/tag.h>
 #include <gst/audio/multichannel.h>
-#include "vorbisenc.h"
+#include <gst/audio/audio.h>
+#include "gstvorbisenc.h"
 
 GST_DEBUG_CATEGORY_EXTERN (vorbisenc_debug);
 #define GST_CAT_DEFAULT vorbisenc_debug
 
-static GstPadTemplate *gst_vorbis_enc_src_template,
-    *gst_vorbis_enc_sink_template;
+static GstStaticPadTemplate vorbis_enc_sink_factory =
+GST_STATIC_PAD_TEMPLATE ("sink",
+    GST_PAD_SINK,
+    GST_PAD_ALWAYS,
+    GST_STATIC_CAPS ("audio/x-raw-float, "
+        "rate = (int) [ 1, 200000 ], "
+        "channels = (int) [ 1, 256 ], " "endianness = (int) BYTE_ORDER, "
+        "width = (int) 32")
+    );
+
+static GstStaticPadTemplate vorbis_enc_src_factory =
+GST_STATIC_PAD_TEMPLATE ("src",
+    GST_PAD_SRC,
+    GST_PAD_ALWAYS,
+    GST_STATIC_CAPS ("audio/x-vorbis")
+    );
+
 
 /* elementfactory information */
 static const GstElementDetails vorbisenc_details =
@@ -134,41 +150,18 @@ gst_vorbis_enc_add_interfaces (GType vorbisenc_type)
       &tag_setter_info);
 }
 
-static GstCaps *
-vorbis_caps_factory (void)
-{
-  return gst_caps_new_simple ("audio/x-vorbis", NULL);
-}
-
-static GstCaps *
-raw_caps_factory (void)
-{
-  /* lowest, highest sample rates come from vorbis/lib/modes/setup_X.h:
-   * 1-200000 Hz */
-  return
-      gst_caps_new_simple ("audio/x-raw-float",
-      "rate", GST_TYPE_INT_RANGE, 1, 200000,
-      "channels", GST_TYPE_INT_RANGE, 1, 256,
-      "endianness", G_TYPE_INT, G_BYTE_ORDER, "width", G_TYPE_INT, 32, NULL);
-}
-
 static void
 gst_vorbis_enc_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
-  GstCaps *raw_caps, *vorbis_caps;
+  GstPadTemplate *src_template, *sink_template;
 
-  raw_caps = raw_caps_factory ();
-  vorbis_caps = vorbis_caps_factory ();
 
-  gst_vorbis_enc_sink_template = gst_pad_template_new ("sink", GST_PAD_SINK,
-      GST_PAD_ALWAYS, raw_caps);
-  gst_vorbis_enc_src_template = gst_pad_template_new ("src", GST_PAD_SRC,
-      GST_PAD_ALWAYS, vorbis_caps);
-  gst_element_class_add_pad_template (element_class,
-      gst_vorbis_enc_sink_template);
-  gst_element_class_add_pad_template (element_class,
-      gst_vorbis_enc_src_template);
+  src_template = gst_static_pad_template_get (&vorbis_enc_src_factory);
+  gst_element_class_add_pad_template (element_class, src_template);
+
+  sink_template = gst_static_pad_template_get (&vorbis_enc_sink_factory);
+  gst_element_class_add_pad_template (element_class, sink_template);
   gst_element_class_set_details (element_class, &vorbisenc_details);
 }
 
@@ -636,7 +629,7 @@ static void
 gst_vorbis_enc_init (GstVorbisEnc * vorbisenc, GstVorbisEncClass * klass)
 {
   vorbisenc->sinkpad =
-      gst_pad_new_from_template (gst_vorbis_enc_sink_template, "sink");
+      gst_pad_new_from_static_template (&vorbis_enc_sink_factory, "sink");
   gst_pad_set_event_function (vorbisenc->sinkpad,
       GST_DEBUG_FUNCPTR (gst_vorbis_enc_sink_event));
   gst_pad_set_chain_function (vorbisenc->sinkpad,
@@ -650,7 +643,7 @@ gst_vorbis_enc_init (GstVorbisEnc * vorbisenc, GstVorbisEncClass * klass)
   gst_element_add_pad (GST_ELEMENT (vorbisenc), vorbisenc->sinkpad);
 
   vorbisenc->srcpad =
-      gst_pad_new_from_template (gst_vorbis_enc_src_template, "src");
+      gst_pad_new_from_static_template (&vorbis_enc_src_factory, "src");
   gst_pad_set_query_function (vorbisenc->srcpad,
       GST_DEBUG_FUNCPTR (gst_vorbis_enc_src_query));
   gst_pad_set_query_type_function (vorbisenc->srcpad,
@@ -901,7 +894,8 @@ gst_vorbis_enc_buffer_from_packet (GstVorbisEnc * vorbisenc,
   /* update the next timestamp, taking granulepos_offset and subgranule offset
    * into account */
   vorbisenc->next_ts =
-      granulepos_to_timestamp_offset (vorbisenc, packet->granulepos);
+      granulepos_to_timestamp_offset (vorbisenc, packet->granulepos) +
+      vorbisenc->initial_ts;
   GST_BUFFER_DURATION (outbuf) =
       vorbisenc->next_ts - GST_BUFFER_TIMESTAMP (outbuf);
 
@@ -910,6 +904,8 @@ gst_vorbis_enc_buffer_from_packet (GstVorbisEnc * vorbisenc,
     vorbisenc->next_discont = FALSE;
   }
 
+  gst_buffer_set_caps (outbuf, vorbisenc->srccaps);
+
   GST_LOG_OBJECT (vorbisenc, "encoded buffer of %d bytes",
       GST_BUFFER_SIZE (outbuf));
   return outbuf;
@@ -1033,6 +1029,26 @@ gst_vorbis_enc_sink_event (GstPad * pad, GstEvent * event)
       }
       res = gst_pad_push_event (vorbisenc->srcpad, event);
       break;
+    case GST_EVENT_NEWSEGMENT:
+    {
+      gboolean update;
+      gdouble rate, applied_rate;
+      GstFormat format;
+      gint64 start, stop, position;
+
+      gst_event_parse_new_segment_full (event, &update, &rate, &applied_rate,
+          &format, &start, &stop, &position);
+      if (format == GST_FORMAT_TIME) {
+        gst_segment_set_newsegment (&vorbisenc->segment, update, rate, format,
+            start, stop, position);
+        if (vorbisenc->initial_ts == GST_CLOCK_TIME_NONE) {
+          GST_DEBUG_OBJECT (vorbisenc, "Initial segment %" GST_SEGMENT_FORMAT,
+              &vorbisenc->segment);
+          vorbisenc->initial_ts = start;
+        }
+      }
+    }
+      /* fall through */
     default:
       res = gst_pad_push_event (vorbisenc->srcpad, event);
       break;
@@ -1042,33 +1058,29 @@ gst_vorbis_enc_sink_event (GstPad * pad, GstEvent * event)
 
 static gboolean
 gst_vorbis_enc_buffer_check_discontinuous (GstVorbisEnc * vorbisenc,
-    GstBuffer * buffer)
+    GstClockTime timestamp, GstClockTime duration)
 {
   gboolean ret = FALSE;
 
-  if (GST_BUFFER_TIMESTAMP (buffer) != GST_CLOCK_TIME_NONE &&
+  if (timestamp != GST_CLOCK_TIME_NONE &&
       vorbisenc->expected_ts != GST_CLOCK_TIME_NONE &&
-      GST_BUFFER_TIMESTAMP (buffer) != vorbisenc->expected_ts) {
+      timestamp + duration != vorbisenc->expected_ts) {
     /* It turns out that a lot of elements don't generate perfect streams due
      * to rounding errors. So, we permit small errors (< 1/2 a sample) without
      * causing a discont.
      */
     int halfsample = GST_SECOND / vorbisenc->frequency / 2;
 
-    if ((GstClockTimeDiff) (GST_BUFFER_TIMESTAMP (buffer) -
-            vorbisenc->expected_ts) > halfsample) {
+    if ((GstClockTimeDiff) (timestamp - vorbisenc->expected_ts) > halfsample) {
       GST_DEBUG_OBJECT (vorbisenc, "Expected TS %" GST_TIME_FORMAT
           ", buffer TS %" GST_TIME_FORMAT,
-          GST_TIME_ARGS (vorbisenc->expected_ts),
-          GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buffer)));
+          GST_TIME_ARGS (vorbisenc->expected_ts), GST_TIME_ARGS (timestamp));
       ret = TRUE;
     }
   }
 
-  if (GST_BUFFER_TIMESTAMP (buffer) != GST_CLOCK_TIME_NONE &&
-      GST_BUFFER_DURATION (buffer) != GST_CLOCK_TIME_NONE) {
-    vorbisenc->expected_ts = GST_BUFFER_TIMESTAMP (buffer) +
-        GST_BUFFER_DURATION (buffer);
+  if (timestamp != GST_CLOCK_TIME_NONE && duration != GST_CLOCK_TIME_NONE) {
+    vorbisenc->expected_ts = timestamp + duration;
   } else
     vorbisenc->expected_ts = GST_CLOCK_TIME_NONE;
 
@@ -1086,12 +1098,26 @@ gst_vorbis_enc_chain (GstPad * pad, GstBuffer * buffer)
   float **vorbis_buffer;
   GstBuffer *buf1, *buf2, *buf3;
   gboolean first = FALSE;
+  GstClockTime timestamp = GST_CLOCK_TIME_NONE;
+  GstClockTime running_time = GST_CLOCK_TIME_NONE;
 
   vorbisenc = GST_VORBISENC (GST_PAD_PARENT (pad));
 
   if (!vorbisenc->setup)
     goto not_setup;
 
+  buffer = gst_audio_buffer_clip (buffer, &vorbisenc->segment,
+      vorbisenc->frequency, 4 * vorbisenc->channels);
+  if (buffer == NULL) {
+    GST_DEBUG_OBJECT (vorbisenc, "Dropping buffer, out of segment");
+    return GST_FLOW_OK;
+  }
+  running_time =
+      gst_segment_to_running_time (&vorbisenc->segment, GST_FORMAT_TIME,
+      GST_BUFFER_TIMESTAMP (buffer));
+  timestamp = running_time + vorbisenc->initial_ts;
+  GST_DEBUG_OBJECT (vorbisenc, "Initial ts is %" GST_TIME_FORMAT,
+      GST_TIME_ARGS (vorbisenc->initial_ts));
   if (!vorbisenc->header_sent) {
     /* Vorbis streams begin with three headers; the initial header (with
        most of the codec setup parameters) which is mandated by the Ogg
@@ -1148,28 +1174,28 @@ gst_vorbis_enc_chain (GstPad * pad, GstBuffer * buffer)
 
     /* now adjust starting granulepos accordingly if the buffer's timestamp is
        nonzero */
-    vorbisenc->next_ts = GST_BUFFER_TIMESTAMP (buffer);
-    vorbisenc->expected_ts = GST_BUFFER_TIMESTAMP (buffer);
+    vorbisenc->next_ts = timestamp;
+    vorbisenc->expected_ts = timestamp;
     vorbisenc->granulepos_offset = gst_util_uint64_scale
-        (GST_BUFFER_TIMESTAMP (buffer), vorbisenc->frequency, GST_SECOND);
+        (running_time, vorbisenc->frequency, GST_SECOND);
     vorbisenc->subgranule_offset = 0;
     vorbisenc->subgranule_offset =
-        vorbisenc->next_ts - granulepos_to_timestamp_offset (vorbisenc, 0);
+        (vorbisenc->next_ts - vorbisenc->initial_ts) -
+        granulepos_to_timestamp_offset (vorbisenc, 0);
 
     vorbisenc->header_sent = TRUE;
     first = TRUE;
   }
 
   if (vorbisenc->expected_ts != GST_CLOCK_TIME_NONE &&
-      GST_BUFFER_TIMESTAMP (buffer) < vorbisenc->expected_ts) {
-    guint64 diff = vorbisenc->expected_ts - GST_BUFFER_TIMESTAMP (buffer);
+      timestamp < vorbisenc->expected_ts) {
+    guint64 diff = vorbisenc->expected_ts - timestamp;
     guint64 diff_bytes;
 
     GST_WARNING_OBJECT (vorbisenc, "Buffer is older than previous "
         "timestamp + duration (%" GST_TIME_FORMAT "< %" GST_TIME_FORMAT
         "), cannot handle. Clipping buffer.",
-        GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buffer)),
-        GST_TIME_ARGS (vorbisenc->expected_ts));
+        GST_TIME_ARGS (timestamp), GST_TIME_ARGS (vorbisenc->expected_ts));
 
     diff_bytes =
         GST_CLOCK_TIME_TO_FRAMES (diff,
@@ -1187,11 +1213,12 @@ gst_vorbis_enc_chain (GstPad * pad, GstBuffer * buffer)
       GST_BUFFER_DURATION (buffer) -= diff;
   }
 
-  if (gst_vorbis_enc_buffer_check_discontinuous (vorbisenc, buffer) && !first) {
-    GST_WARNING_OBJECT (vorbisenc, "Buffer is discontinuous, flushing encoder "
-        "and restarting (Discont from %" GST_TIME_FORMAT
-        " to %" GST_TIME_FORMAT ")", GST_TIME_ARGS (vorbisenc->next_ts),
-        GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buffer)));
+  if (gst_vorbis_enc_buffer_check_discontinuous (vorbisenc, timestamp,
+          GST_BUFFER_DURATION (buffer)) && !first) {
+    GST_WARNING_OBJECT (vorbisenc,
+        "Buffer is discontinuous, flushing encoder "
+        "and restarting (Discont from %" GST_TIME_FORMAT " to %" GST_TIME_FORMAT
+        ")", GST_TIME_ARGS (vorbisenc->next_ts), GST_TIME_ARGS (timestamp));
     /* Re-initialise encoder (there's unfortunately no API to flush it) */
     if ((ret = gst_vorbis_enc_clear (vorbisenc)) != GST_FLOW_OK)
       return ret;
@@ -1200,11 +1227,11 @@ gst_vorbis_enc_chain (GstPad * pad, GstBuffer * buffer)
                                    we successfully initialised earlier */
 
     /* Now, set our granulepos offset appropriately. */
-    vorbisenc->next_ts = GST_BUFFER_TIMESTAMP (buffer);
+    vorbisenc->next_ts = timestamp;
     /* We need to round to the nearest whole number of samples, not just do
      * a truncating division here */
     vorbisenc->granulepos_offset = gst_util_uint64_scale
-        (GST_BUFFER_TIMESTAMP (buffer) + GST_SECOND / vorbisenc->frequency / 2
+        (running_time + GST_SECOND / vorbisenc->frequency / 2
         - vorbisenc->subgranule_offset, vorbisenc->frequency, GST_SECOND);
 
     vorbisenc->header_sent = TRUE;
@@ -1418,6 +1445,8 @@ gst_vorbis_enc_change_state (GstElement * element, GstStateChange transition)
       vorbisenc->setup = FALSE;
       vorbisenc->next_discont = FALSE;
       vorbisenc->header_sent = FALSE;
+      gst_segment_init (&vorbisenc->segment, GST_FORMAT_TIME);
+      vorbisenc->initial_ts = GST_CLOCK_TIME_NONE;
       break;
     case GST_STATE_CHANGE_PAUSED_TO_PLAYING:
       break;
