@@ -18,7 +18,15 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
-
+/**
+ * SECTION:gstpropertyprobe
+ * @short_description: Interface for probing possible property values
+ *
+ * The property probe is a way to autodetect allowed values for a GObject
+ * property. It's primary use is to autodetect device-names in several elements.
+ *
+ * The interface is implemented by many hardware sources and sinks.
+ */
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -69,6 +77,15 @@ gst_property_probe_iface_init (GstPropertyProbeInterface * iface)
   static gboolean initialized = FALSE;
 
   if (!initialized) {
+    /**
+     * GstPropertyProbe::probe-needed
+     * @pspec: #GParamSpec that needs a probe
+     *
+     */
+    /* FIXME:
+     * what is the purpose of this signal, I can't find any usage of it
+     * according to proto n *.h, it should be g_cclosure_marshal_VOID__PARAM
+     */
     gst_property_probe_signals[SIGNAL_PROBE_NEEDED] =
         g_signal_new ("probe-needed", G_TYPE_FROM_CLASS (iface),
         G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (GstPropertyProbeInterface,
@@ -88,16 +105,16 @@ gst_property_probe_iface_init (GstPropertyProbeInterface * iface)
  *
  * Get a list of properties for which probing is supported.
  *
- * Returns the list of properties for which probing is supported
+ * Returns: the list of properties for which probing is supported
  * by this element.
  */
-
 const GList *
 gst_property_probe_get_properties (GstPropertyProbe * probe)
 {
   GstPropertyProbeInterface *iface;
 
   g_return_val_if_fail (probe != NULL, NULL);
+  g_return_val_if_fail (GST_IS_PROPERTY_PROBE (probe), NULL);
 
   iface = GST_PROPERTY_PROBE_GET_IFACE (probe);
 
@@ -107,19 +124,35 @@ gst_property_probe_get_properties (GstPropertyProbe * probe)
   return NULL;
 }
 
+/**
+ * gst_property_probe_get_property:
+ * @probe: the #GstPropertyProbe to get the properties for.
+ * @name: name of the property.
+ *
+ * Get #GParamSpec for a property for which probing is supported.
+ *
+ * Returns: the #GParamSpec of %NULL.
+ */
 const GParamSpec *
 gst_property_probe_get_property (GstPropertyProbe * probe, const gchar * name)
 {
-  const GList *pspecs = gst_property_probe_get_properties (probe);
+  const GList *pspecs;
 
   g_return_val_if_fail (probe != NULL, NULL);
+  g_return_val_if_fail (GST_IS_PROPERTY_PROBE (probe), NULL);
   g_return_val_if_fail (name != NULL, NULL);
 
+  pspecs = gst_property_probe_get_properties (probe);
+
   while (pspecs) {
     const GParamSpec *pspec = pspecs->data;
 
-    if (!strcmp (pspec->name, name))
-      return pspec;
+    if (pspec) {
+      if (!strcmp (pspec->name, name))
+        return pspec;
+    } else {
+      GST_WARNING_OBJECT (probe, "NULL paramspec in property probe list");
+    }
 
     pspecs = pspecs->next;
   }
@@ -127,6 +160,13 @@ gst_property_probe_get_property (GstPropertyProbe * probe, const gchar * name)
   return NULL;
 }
 
+/**
+ * gst_property_probe_probe_property:
+ * @probe: the #GstPropertyProbe to check.
+ * @pspec: #GParamSpec of the property.
+ *
+ * Runs a probe on the property specified by %pspec
+ */
 void
 gst_property_probe_probe_property (GstPropertyProbe * probe,
     const GParamSpec * pspec)
@@ -134,6 +174,7 @@ gst_property_probe_probe_property (GstPropertyProbe * probe,
   GstPropertyProbeInterface *iface;
 
   g_return_if_fail (probe != NULL);
+  g_return_if_fail (GST_IS_PROPERTY_PROBE (probe));
   g_return_if_fail (pspec != NULL);
 
   iface = GST_PROPERTY_PROBE_GET_IFACE (probe);
@@ -145,15 +186,10 @@ gst_property_probe_probe_property (GstPropertyProbe * probe,
 /**
  * gst_property_probe_probe_property_name:
  * @probe: the #GstPropertyProbe to check.
- * @name: name of the property to return.
+ * @name: name of the property.
  *
- * Returns the #GParamSpec for the given property. It's similar to
- * g_object_class_find_property (), except that this function only
- * takes "probe'able" properties into account.
- *
- * Returns: the #GParamSpec that belongs to the given property.
+ * Runs a probe on the property specified by %name.
  */
-
 void
 gst_property_probe_probe_property_name (GstPropertyProbe * probe,
     const gchar * name)
@@ -161,6 +197,7 @@ gst_property_probe_probe_property_name (GstPropertyProbe * probe,
   const GParamSpec *pspec;
 
   g_return_if_fail (probe != NULL);
+  g_return_if_fail (GST_IS_PROPERTY_PROBE (probe));
   g_return_if_fail (name != NULL);
 
   pspec = g_object_class_find_property (G_OBJECT_GET_CLASS (probe), name);
@@ -185,7 +222,6 @@ gst_property_probe_probe_property_name (GstPropertyProbe * probe,
  *
  * Returns: TRUE if the property needs a new probe, FALSE if not.
  */
-
 gboolean
 gst_property_probe_needs_probe (GstPropertyProbe * probe,
     const GParamSpec * pspec)
@@ -193,6 +229,7 @@ gst_property_probe_needs_probe (GstPropertyProbe * probe,
   GstPropertyProbeInterface *iface;
 
   g_return_val_if_fail (probe != NULL, FALSE);
+  g_return_val_if_fail (GST_IS_PROPERTY_PROBE (probe), FALSE);
   g_return_val_if_fail (pspec != NULL, FALSE);
 
   iface = GST_PROPERTY_PROBE_GET_IFACE (probe);
@@ -212,7 +249,6 @@ gst_property_probe_needs_probe (GstPropertyProbe * probe,
  *
  * Returns: TRUE if the property needs a new probe, FALSE if not.
  */
-
 gboolean
 gst_property_probe_needs_probe_name (GstPropertyProbe * probe,
     const gchar * name)
@@ -220,6 +256,7 @@ gst_property_probe_needs_probe_name (GstPropertyProbe * probe,
   const GParamSpec *pspec;
 
   g_return_val_if_fail (probe != NULL, FALSE);
+  g_return_val_if_fail (GST_IS_PROPERTY_PROBE (probe), FALSE);
   g_return_val_if_fail (name != NULL, FALSE);
 
   pspec = g_object_class_find_property (G_OBJECT_GET_CLASS (probe), name);
@@ -241,7 +278,6 @@ gst_property_probe_needs_probe_name (GstPropertyProbe * probe,
  *
  * Returns: A list of valid values for the given property.
  */
-
 GValueArray *
 gst_property_probe_get_values (GstPropertyProbe * probe,
     const GParamSpec * pspec)
@@ -249,6 +285,7 @@ gst_property_probe_get_values (GstPropertyProbe * probe,
   GstPropertyProbeInterface *iface;
 
   g_return_val_if_fail (probe != NULL, NULL);
+  g_return_val_if_fail (GST_IS_PROPERTY_PROBE (probe), NULL);
   g_return_val_if_fail (pspec != NULL, NULL);
 
   iface = GST_PROPERTY_PROBE_GET_IFACE (probe);
@@ -268,7 +305,6 @@ gst_property_probe_get_values (GstPropertyProbe * probe,
  *
  * Returns: A list of valid values for the given property.
  */
-
 GValueArray *
 gst_property_probe_get_values_name (GstPropertyProbe * probe,
     const gchar * name)
@@ -276,6 +312,7 @@ gst_property_probe_get_values_name (GstPropertyProbe * probe,
   const GParamSpec *pspec;
 
   g_return_val_if_fail (probe != NULL, NULL);
+  g_return_val_if_fail (GST_IS_PROPERTY_PROBE (probe), NULL);
   g_return_val_if_fail (name != NULL, NULL);
 
   pspec = g_object_class_find_property (G_OBJECT_GET_CLASS (probe), name);
@@ -298,7 +335,6 @@ gst_property_probe_get_values_name (GstPropertyProbe * probe,
  *
  * Returns: the list of valid values for this property.
  */
-
 GValueArray *
 gst_property_probe_probe_and_get_values (GstPropertyProbe * probe,
     const GParamSpec * pspec)
@@ -306,6 +342,7 @@ gst_property_probe_probe_and_get_values (GstPropertyProbe * probe,
   GstPropertyProbeInterface *iface;
 
   g_return_val_if_fail (probe != NULL, NULL);
+  g_return_val_if_fail (GST_IS_PROPERTY_PROBE (probe), NULL);
   g_return_val_if_fail (pspec != NULL, NULL);
 
   iface = GST_PROPERTY_PROBE_GET_IFACE (probe);
@@ -325,7 +362,6 @@ gst_property_probe_probe_and_get_values (GstPropertyProbe * probe,
  *
  * Returns: the list of valid values for this property.
  */
-
 GValueArray *
 gst_property_probe_probe_and_get_values_name (GstPropertyProbe * probe,
     const gchar * name)
@@ -333,6 +369,7 @@ gst_property_probe_probe_and_get_values_name (GstPropertyProbe * probe,
   const GParamSpec *pspec;
 
   g_return_val_if_fail (probe != NULL, NULL);
+  g_return_val_if_fail (GST_IS_PROPERTY_PROBE (probe), NULL);
   g_return_val_if_fail (name != NULL, NULL);
 
   pspec = g_object_class_find_property (G_OBJECT_GET_CLASS (probe), name);
