@@ -580,16 +580,6 @@ gst_video_format_from_fourcc (guint32 fourcc)
       return GST_VIDEO_FORMAT_Y41B;
     case GST_MAKE_FOURCC ('Y', '4', '2', 'B'):
       return GST_VIDEO_FORMAT_Y42B;
-    case GST_MAKE_FOURCC ('Y', '4', '4', '4'):
-      return GST_VIDEO_FORMAT_Y444;
-    case GST_MAKE_FOURCC ('v', '2', '1', '0'):
-      return GST_VIDEO_FORMAT_v210;
-    case GST_MAKE_FOURCC ('v', '2', '1', '6'):
-      return GST_VIDEO_FORMAT_v216;
-    case GST_MAKE_FOURCC ('N', 'V', '1', '2'):
-      return GST_VIDEO_FORMAT_NV12;
-    case GST_MAKE_FOURCC ('N', 'V', '2', '1'):
-      return GST_VIDEO_FORMAT_NV21;
     default:
       return GST_VIDEO_FORMAT_UNKNOWN;
   }
@@ -629,16 +619,6 @@ gst_video_format_to_fourcc (GstVideoFormat format)
       return GST_MAKE_FOURCC ('Y', '4', '1', 'B');
     case GST_VIDEO_FORMAT_Y42B:
       return GST_MAKE_FOURCC ('Y', '4', '2', 'B');
-    case GST_VIDEO_FORMAT_Y444:
-      return GST_MAKE_FOURCC ('Y', '4', '4', '4');
-    case GST_VIDEO_FORMAT_v210:
-      return GST_MAKE_FOURCC ('v', '2', '1', '0');
-    case GST_VIDEO_FORMAT_v216:
-      return GST_MAKE_FOURCC ('v', '2', '1', '6');
-    case GST_VIDEO_FORMAT_NV12:
-      return GST_MAKE_FOURCC ('N', 'V', '1', '2');
-    case GST_VIDEO_FORMAT_NV21:
-      return GST_MAKE_FOURCC ('N', 'V', '2', '1');
     default:
       return 0;
   }
@@ -739,11 +719,6 @@ gst_video_format_is_rgb (GstVideoFormat format)
     case GST_VIDEO_FORMAT_AYUV:
     case GST_VIDEO_FORMAT_Y41B:
     case GST_VIDEO_FORMAT_Y42B:
-    case GST_VIDEO_FORMAT_Y444:
-    case GST_VIDEO_FORMAT_v210:
-    case GST_VIDEO_FORMAT_v216:
-    case GST_VIDEO_FORMAT_NV12:
-    case GST_VIDEO_FORMAT_NV21:
       return FALSE;
     case GST_VIDEO_FORMAT_RGBx:
     case GST_VIDEO_FORMAT_BGRx:
@@ -783,11 +758,6 @@ gst_video_format_is_yuv (GstVideoFormat format)
     case GST_VIDEO_FORMAT_AYUV:
     case GST_VIDEO_FORMAT_Y41B:
     case GST_VIDEO_FORMAT_Y42B:
-    case GST_VIDEO_FORMAT_Y444:
-    case GST_VIDEO_FORMAT_v210:
-    case GST_VIDEO_FORMAT_v216:
-    case GST_VIDEO_FORMAT_NV12:
-    case GST_VIDEO_FORMAT_NV21:
       return TRUE;
     case GST_VIDEO_FORMAT_RGBx:
     case GST_VIDEO_FORMAT_BGRx:
@@ -827,11 +797,6 @@ gst_video_format_has_alpha (GstVideoFormat format)
     case GST_VIDEO_FORMAT_UYVY:
     case GST_VIDEO_FORMAT_Y41B:
     case GST_VIDEO_FORMAT_Y42B:
-    case GST_VIDEO_FORMAT_Y444:
-    case GST_VIDEO_FORMAT_v210:
-    case GST_VIDEO_FORMAT_v216:
-    case GST_VIDEO_FORMAT_NV12:
-    case GST_VIDEO_FORMAT_NV21:
       return FALSE;
     case GST_VIDEO_FORMAT_AYUV:
     case GST_VIDEO_FORMAT_RGBA:
@@ -906,7 +871,7 @@ gst_video_format_get_row_stride (GstVideoFormat format, int component,
       if (component == 0) {
         return GST_ROUND_UP_4 (width);
       } else {
-        return GST_ROUND_UP_16 (width) / 4;
+        return GST_ROUND_UP_8 (width) / 4;
       }
     case GST_VIDEO_FORMAT_Y42B:
       if (component == 0) {
@@ -914,15 +879,6 @@ gst_video_format_get_row_stride (GstVideoFormat format, int component,
       } else {
         return GST_ROUND_UP_8 (width) / 2;
       }
-    case GST_VIDEO_FORMAT_Y444:
-      return GST_ROUND_UP_4 (width);
-    case GST_VIDEO_FORMAT_v210:
-      return ((width + 47) / 48) * 128;
-    case GST_VIDEO_FORMAT_v216:
-      return GST_ROUND_UP_8 (width * 4);
-    case GST_VIDEO_FORMAT_NV12:
-    case GST_VIDEO_FORMAT_NV21:
-      return GST_ROUND_UP_4 (width);
     default:
       return 0;
   }
@@ -953,7 +909,6 @@ gst_video_format_get_pixel_stride (GstVideoFormat format, int component)
     case GST_VIDEO_FORMAT_YV12:
     case GST_VIDEO_FORMAT_Y41B:
     case GST_VIDEO_FORMAT_Y42B:
-    case GST_VIDEO_FORMAT_Y444:
       return 1;
     case GST_VIDEO_FORMAT_YUY2:
     case GST_VIDEO_FORMAT_YVYU:
@@ -976,22 +931,6 @@ gst_video_format_get_pixel_stride (GstVideoFormat format, int component)
     case GST_VIDEO_FORMAT_RGB:
     case GST_VIDEO_FORMAT_BGR:
       return 3;
-    case GST_VIDEO_FORMAT_v210:
-      /* v210 is packed at the bit level, so pixel stride doesn't make sense */
-      return 0;
-    case GST_VIDEO_FORMAT_v216:
-      if (component == 0) {
-        return 4;
-      } else {
-        return 8;
-      }
-    case GST_VIDEO_FORMAT_NV12:
-    case GST_VIDEO_FORMAT_NV21:
-      if (component == 0) {
-        return 1;
-      } else {
-        return 2;
-      }
     default:
       return 0;
   }
@@ -1025,19 +964,22 @@ gst_video_format_get_component_width (GstVideoFormat format, int component,
     case GST_VIDEO_FORMAT_YUY2:
     case GST_VIDEO_FORMAT_YVYU:
     case GST_VIDEO_FORMAT_UYVY:
-    case GST_VIDEO_FORMAT_Y42B:
-    case GST_VIDEO_FORMAT_v210:
-    case GST_VIDEO_FORMAT_v216:
       if (component == 0) {
         return width;
       } else {
         return GST_ROUND_UP_2 (width) / 2;
       }
-    case GST_VIDEO_FORMAT_Y41B:
+    case GST_VIDEO_FORMAT_Y41B:        /* CHECKME: component_width for Y41B */
       if (component == 0) {
         return width;
       } else {
-        return GST_ROUND_UP_4 (width) / 4;
+        return GST_ROUND_UP_8 (width) / 4;
+      }
+    case GST_VIDEO_FORMAT_Y42B:        /* CHECKME: component_width for Y42B */
+      if (component == 0) {
+        return width;
+      } else {
+        return GST_ROUND_UP_8 (width) / 2;
       }
     case GST_VIDEO_FORMAT_AYUV:
     case GST_VIDEO_FORMAT_RGBx:
@@ -1050,9 +992,6 @@ gst_video_format_get_component_width (GstVideoFormat format, int component,
     case GST_VIDEO_FORMAT_ABGR:
     case GST_VIDEO_FORMAT_RGB:
     case GST_VIDEO_FORMAT_BGR:
-    case GST_VIDEO_FORMAT_Y444:
-    case GST_VIDEO_FORMAT_NV12:
-    case GST_VIDEO_FORMAT_NV21:
       return width;
     default:
       return 0;
@@ -1084,8 +1023,6 @@ gst_video_format_get_component_height (GstVideoFormat format, int component,
   switch (format) {
     case GST_VIDEO_FORMAT_I420:
     case GST_VIDEO_FORMAT_YV12:
-    case GST_VIDEO_FORMAT_NV12:
-    case GST_VIDEO_FORMAT_NV21:
       if (component == 0) {
         return height;
       } else {
@@ -1107,9 +1044,6 @@ gst_video_format_get_component_height (GstVideoFormat format, int component,
     case GST_VIDEO_FORMAT_ABGR:
     case GST_VIDEO_FORMAT_RGB:
     case GST_VIDEO_FORMAT_BGR:
-    case GST_VIDEO_FORMAT_Y444:
-    case GST_VIDEO_FORMAT_v210:
-    case GST_VIDEO_FORMAT_v216:
       return height;
     default:
       return 0;
@@ -1265,8 +1199,7 @@ gst_video_format_get_component_offset (GstVideoFormat format, int component,
       if (component == 1)
         return GST_ROUND_UP_4 (width) * height;
       if (component == 2)
-        return (GST_ROUND_UP_4 (width) +
-            (GST_ROUND_UP_16 (width) / 4)) * height;
+        return (GST_ROUND_UP_4 (width) + (GST_ROUND_UP_8 (width) / 4)) * height;
       return 0;
     case GST_VIDEO_FORMAT_Y42B:
       if (component == 0)
@@ -1276,33 +1209,6 @@ gst_video_format_get_component_offset (GstVideoFormat format, int component,
       if (component == 2)
         return (GST_ROUND_UP_4 (width) + (GST_ROUND_UP_8 (width) / 2)) * height;
       return 0;
-    case GST_VIDEO_FORMAT_Y444:
-      return GST_ROUND_UP_4 (width) * height * component;
-    case GST_VIDEO_FORMAT_v210:
-      /* v210 is bit-packed, so this doesn't make sense */
-      return 0;
-    case GST_VIDEO_FORMAT_v216:
-      if (component == 0)
-        return 0;
-      if (component == 1)
-        return 2;
-      if (component == 2)
-        return 6;
-      return 0;
-    case GST_VIDEO_FORMAT_NV12:
-      if (component == 0)
-        return 0;
-      if (component == 1)
-        return GST_ROUND_UP_4 (width) * GST_ROUND_UP_2 (height);
-      if (component == 2)
-        return GST_ROUND_UP_4 (width) * GST_ROUND_UP_2 (height) + 1;
-    case GST_VIDEO_FORMAT_NV21:
-      if (component == 0)
-        return 0;
-      if (component == 1)
-        return GST_ROUND_UP_4 (width) * GST_ROUND_UP_2 (height) + 1;
-      if (component == 2)
-        return GST_ROUND_UP_4 (width) * GST_ROUND_UP_2 (height);
     default:
       return 0;
   }
@@ -1355,19 +1261,10 @@ gst_video_format_get_size (GstVideoFormat format, int width, int height)
       return GST_ROUND_UP_4 (width * 3) * height;
     case GST_VIDEO_FORMAT_Y41B:
       /* simplification of ROUNDUP4(w)*h + 2*((ROUNDUP8(w)/4)*h */
-      return (GST_ROUND_UP_4 (width) + (GST_ROUND_UP_16 (width) / 2)) * height;
+      return (GST_ROUND_UP_4 (width) + (GST_ROUND_UP_8 (width) / 2)) * height;
     case GST_VIDEO_FORMAT_Y42B:
       /* simplification of ROUNDUP4(w)*h + 2*(ROUNDUP8(w)/2)*h: */
       return (GST_ROUND_UP_4 (width) + GST_ROUND_UP_8 (width)) * height;
-    case GST_VIDEO_FORMAT_Y444:
-      return GST_ROUND_UP_4 (width) * height * 3;
-    case GST_VIDEO_FORMAT_v210:
-      return ((width + 47) / 48) * 128 * height;
-    case GST_VIDEO_FORMAT_v216:
-      return GST_ROUND_UP_8 (width * 4) * height;
-    case GST_VIDEO_FORMAT_NV12:
-    case GST_VIDEO_FORMAT_NV21:
-      return GST_ROUND_UP_4 (width) * GST_ROUND_UP_2 (height) * 3 / 2;
     default:
       return 0;
   }
@@ -1500,71 +1397,3 @@ done:
 
   return ret;
 }
-
-#define GST_VIDEO_EVENT_STILL_STATE_NAME "GstEventStillFrame"
-
-/**
- * gst_video_event_new_still_frame:
- * @in_still: boolean value for the still-frame state of the event.
- *
- * Creates a new Still Frame event. If @in_still is %TRUE, then the event
- * represents the start of a still frame sequence. If it is %FALSE, then
- * the event ends a still frame sequence.
- *
- * To parse an event created by gst_video_event_new_still_frame() use
- * gst_video_event_parse_still_frame().
- *
- * Returns: The new GstEvent
- * Since: 0.10.26
- */
-GstEvent *
-gst_video_event_new_still_frame (gboolean in_still)
-{
-  GstEvent *still_event;
-  GstStructure *s;
-
-  s = gst_structure_new (GST_VIDEO_EVENT_STILL_STATE_NAME,
-      "still-state", G_TYPE_BOOLEAN, in_still, NULL);
-  still_event = gst_event_new_custom (GST_EVENT_CUSTOM_DOWNSTREAM, s);
-
-  return still_event;
-}
-
-/**
- * gst_video_event_parse_still_frame:
- * @event: A #GstEvent to parse
- * @in_still: A boolean to receive the still-frame status from the event, or NULL
- *
- * Parse a #GstEvent, identify if it is a Still Frame event, and
- * return the still-frame state from the event if it is.
- * If the event represents the start of a still frame, the in_still
- * variable will be set to TRUE, otherwise FALSE. It is OK to pass NULL for the
- * in_still variable order to just check whether the event is a valid still-frame
- * event.
- *
- * Create a still frame event using gst_video_event_new_still_frame()
- *
- * Returns: %TRUE if the event is a valid still-frame event. %FALSE if not
- * Since: 0.10.26
- */
-gboolean
-gst_video_event_parse_still_frame (GstEvent * event, gboolean * in_still)
-{
-  const GstStructure *s;
-  gboolean ev_still_state;
-
-  g_return_val_if_fail (event != NULL, FALSE);
-
-  if (GST_EVENT_TYPE (event) != GST_EVENT_CUSTOM_DOWNSTREAM)
-    return FALSE;               /* Not a still frame event */
-
-  s = gst_event_get_structure (event);
-  if (s == NULL
-      || !gst_structure_has_name (s, GST_VIDEO_EVENT_STILL_STATE_NAME))
-    return FALSE;               /* Not a still frame event */
-  if (!gst_structure_get_boolean (s, "still-state", &ev_still_state))
-    return FALSE;               /* Not a still frame event */
-  if (in_still)
-    *in_still = ev_still_state;
-  return TRUE;
-}
