@@ -231,8 +231,8 @@ gst_ximage_buffer_finalize (GstXImageBuffer * ximage)
   }
 
   if (!recycled)
-    GST_MINI_OBJECT_CLASS (ximage_buffer_parent_class)->
-        finalize (GST_MINI_OBJECT (ximage));
+    GST_MINI_OBJECT_CLASS (ximage_buffer_parent_class)->finalize
+        (GST_MINI_OBJECT (ximage));
 
 beach:
   return;
@@ -701,7 +701,7 @@ gst_ximagesink_ximage_put (GstXImageSink * ximagesink, GstXImageBuffer * ximage)
   if (ximage && ximagesink->cur_image != ximage) {
     if (ximagesink->cur_image) {
       GST_LOG_OBJECT (ximagesink, "unreffing %p", ximagesink->cur_image);
-      gst_buffer_unref (ximagesink->cur_image);
+      gst_buffer_unref (GST_BUFFER_CAST (ximagesink->cur_image));
     }
     GST_LOG_OBJECT (ximagesink, "reffing %p as our current image", ximage);
     ximagesink->cur_image =
@@ -1364,8 +1364,8 @@ gst_ximagesink_getcaps (GstBaseSink * bsink)
 
   /* get a template copy and add the pixel aspect ratio */
   caps =
-      gst_caps_copy (gst_pad_get_pad_template_caps (GST_BASE_SINK (ximagesink)->
-          sinkpad));
+      gst_caps_copy (gst_pad_get_pad_template_caps (GST_BASE_SINK
+          (ximagesink)->sinkpad));
   for (i = 0; i < gst_caps_get_size (caps); ++i) {
     GstStructure *structure = gst_caps_get_structure (caps, i);
 
@@ -1659,6 +1659,7 @@ gst_ximagesink_buffer_alloc (GstBaseSink * bsink, guint64 offset, guint size,
   GstCaps *alloc_caps;
   gboolean alloc_unref = FALSE;
   gint width, height;
+  GstVideoRectangle dst, src, result;
 
   ximagesink = GST_XIMAGESINK (bsink);
 
@@ -1674,93 +1675,90 @@ gst_ximagesink_buffer_alloc (GstBaseSink * bsink, guint64 offset, guint size,
 
   /* get struct to see what is requested */
   structure = gst_caps_get_structure (caps, 0);
+  if (!gst_structure_get_int (structure, "width", &width) ||
+      !gst_structure_get_int (structure, "height", &height)) {
+    GST_WARNING_OBJECT (ximagesink, "invalid caps for buffer allocation %"
+        GST_PTR_FORMAT, caps);
+    ret = GST_FLOW_NOT_NEGOTIATED;
+    goto beach;
+  }
 
-  if (gst_structure_get_int (structure, "width", &width) &&
-      gst_structure_get_int (structure, "height", &height)) {
-    GstVideoRectangle dst, src, result;
-
-    src.w = width;
-    src.h = height;
-
-    /* We take the flow_lock because the window might go away */
-    g_mutex_lock (ximagesink->flow_lock);
-    if (!ximagesink->xwindow) {
-      g_mutex_unlock (ximagesink->flow_lock);
-      goto alloc;
-    }
-
-    /* What is our geometry */
-    gst_ximagesink_xwindow_update_geometry (ximagesink, ximagesink->xwindow);
-    dst.w = ximagesink->xwindow->width;
-    dst.h = ximagesink->xwindow->height;
+  src.w = width;
+  src.h = height;
 
+  /* We take the flow_lock because the window might go away */
+  g_mutex_lock (ximagesink->flow_lock);
+  if (!ximagesink->xwindow) {
     g_mutex_unlock (ximagesink->flow_lock);
+    goto alloc;
+  }
 
-    if (ximagesink->keep_aspect) {
-      GST_LOG_OBJECT (ximagesink, "enforcing aspect ratio in reverse caps "
-          "negotiation");
-      gst_video_sink_center_rect (src, dst, &result, TRUE);
-    } else {
-      GST_LOG_OBJECT (ximagesink, "trying to resize to window geometry "
-          "ignoring aspect ratio");
-      result.x = result.y = 0;
-      result.w = dst.w;
-      result.h = dst.h;
+  /* What is our geometry */
+  gst_ximagesink_xwindow_update_geometry (ximagesink, ximagesink->xwindow);
+  dst.w = ximagesink->xwindow->width;
+  dst.h = ximagesink->xwindow->height;
+
+  g_mutex_unlock (ximagesink->flow_lock);
+
+  if (ximagesink->keep_aspect) {
+    GST_LOG_OBJECT (ximagesink, "enforcing aspect ratio in reverse caps "
+        "negotiation");
+    gst_video_sink_center_rect (src, dst, &result, TRUE);
+  } else {
+    GST_LOG_OBJECT (ximagesink, "trying to resize to window geometry "
+        "ignoring aspect ratio");
+    result.x = result.y = 0;
+    result.w = dst.w;
+    result.h = dst.h;
+  }
+
+  /* We would like another geometry */
+  if (width != result.w || height != result.h) {
+    int nom, den;
+    GstCaps *desired_caps;
+    GstStructure *desired_struct;
+
+    /* make a copy of the incomming caps to create the new
+     * suggestion. We can't use make_writable because we might
+     * then destroy the original caps which we still need when the
+     * peer does not accept the suggestion. */
+    desired_caps = gst_caps_copy (caps);
+    desired_struct = gst_caps_get_structure (desired_caps, 0);
+
+    GST_DEBUG ("we would love to receive a %dx%d video", result.w, result.h);
+    gst_structure_set (desired_struct, "width", G_TYPE_INT, result.w, NULL);
+    gst_structure_set (desired_struct, "height", G_TYPE_INT, result.h, NULL);
+
+    /* PAR property overrides the X calculated one */
+    if (ximagesink->par) {
+      nom = gst_value_get_fraction_numerator (ximagesink->par);
+      den = gst_value_get_fraction_denominator (ximagesink->par);
+      gst_structure_set (desired_struct, "pixel-aspect-ratio",
+          GST_TYPE_FRACTION, nom, den, NULL);
+    } else if (ximagesink->xcontext->par) {
+      nom = gst_value_get_fraction_numerator (ximagesink->xcontext->par);
+      den = gst_value_get_fraction_denominator (ximagesink->xcontext->par);
+      gst_structure_set (desired_struct, "pixel-aspect-ratio",
+          GST_TYPE_FRACTION, nom, den, NULL);
     }
 
-    /* We would like another geometry */
-    if (width != result.w || height != result.h) {
-      int nom, den;
-      GstCaps *desired_caps;
-      GstStructure *desired_struct;
-
-      /* make a copy of the incomming caps to create the new
-       * suggestion. We can't use make_writable because we might
-       * then destroy the original caps which we still need when the
-       * peer does not accept the suggestion. */
-      desired_caps = gst_caps_copy (caps);
-      desired_struct = gst_caps_get_structure (desired_caps, 0);
-
-      GST_DEBUG ("we would love to receive a %dx%d video", result.w, result.h);
-      gst_structure_set (desired_struct, "width", G_TYPE_INT, result.w, NULL);
-      gst_structure_set (desired_struct, "height", G_TYPE_INT, result.h, NULL);
-
-      /* PAR property overrides the X calculated one */
-      if (ximagesink->par) {
-        nom = gst_value_get_fraction_numerator (ximagesink->par);
-        den = gst_value_get_fraction_denominator (ximagesink->par);
-        gst_structure_set (desired_struct, "pixel-aspect-ratio",
-            GST_TYPE_FRACTION, nom, den, NULL);
-      } else if (ximagesink->xcontext->par) {
-        nom = gst_value_get_fraction_numerator (ximagesink->xcontext->par);
-        den = gst_value_get_fraction_denominator (ximagesink->xcontext->par);
-        gst_structure_set (desired_struct, "pixel-aspect-ratio",
-            GST_TYPE_FRACTION, nom, den, NULL);
-      }
-
-      /* see if peer accepts our new suggestion, if there is no peer, this 
-       * function returns true. */
-      if (gst_pad_peer_accept_caps (GST_VIDEO_SINK_PAD (ximagesink),
-              desired_caps)) {
-        gint bpp;
-
-        bpp = size / height / width;
-        /* we will not alloc a buffer of the new suggested caps. Make sure
-         * we also unref this new caps after we set it on the buffer. */
-        alloc_caps = desired_caps;
-        alloc_unref = TRUE;
-        width = result.w;
-        height = result.h;
-        size = bpp * width * height;
-        GST_DEBUG ("peer pad accepts our desired caps %" GST_PTR_FORMAT
-            " buffer size is now %d bytes", desired_caps, size);
-      } else {
-        GST_DEBUG ("peer pad does not accept our desired caps %" GST_PTR_FORMAT,
-            desired_caps);
-        /* we alloc a buffer with the original incomming caps */
-        width = GST_VIDEO_SINK_WIDTH (ximagesink);
-        height = GST_VIDEO_SINK_HEIGHT (ximagesink);
-      }
+    /* see if peer accepts our new suggestion, if there is no peer, this 
+     * function returns true. */
+    if (gst_pad_peer_accept_caps (GST_VIDEO_SINK_PAD (ximagesink),
+            desired_caps)) {
+      /* we will not alloc a buffer of the new suggested caps. Make sure
+       * we also unref this new caps after we set it on the buffer. */
+      alloc_caps = desired_caps;
+      alloc_unref = TRUE;
+      width = result.w;
+      height = result.h;
+      GST_DEBUG ("peer pad accepts our desired caps %" GST_PTR_FORMAT,
+          desired_caps);
+    } else {
+      GST_DEBUG ("peer pad does not accept our desired caps %" GST_PTR_FORMAT,
+          desired_caps);
+      /* we alloc a buffer with the original incomming caps already in the
+       * width and height variables */
     }
   }
 
@@ -1804,6 +1802,7 @@ alloc:
 
   *buf = GST_BUFFER_CAST (ximage);
 
+beach:
   return ret;
 }
 
@@ -1853,11 +1852,11 @@ gst_ximagesink_navigation_send_event (GstNavigation * navigation,
 
   g_mutex_unlock (ximagesink->flow_lock);
 
-  if (gst_structure_get_double (structure, "pointer_x", &x)) {
+  if (x_offset > 0 && gst_structure_get_double (structure, "pointer_x", &x)) {
     x -= x_offset / 2;
     gst_structure_set (structure, "pointer_x", G_TYPE_DOUBLE, x, NULL);
   }
-  if (gst_structure_get_double (structure, "pointer_y", &y)) {
+  if (y_offset > 0 && gst_structure_get_double (structure, "pointer_y", &y)) {
     y -= y_offset / 2;
     gst_structure_set (structure, "pointer_y", G_TYPE_DOUBLE, y, NULL);
   }
@@ -2117,11 +2116,11 @@ gst_ximagesink_reset (GstXImageSink * ximagesink)
     g_thread_join (thread);
 
   if (ximagesink->ximage) {
-    gst_buffer_unref (ximagesink->ximage);
+    gst_buffer_unref (GST_BUFFER_CAST (ximagesink->ximage));
     ximagesink->ximage = NULL;
   }
   if (ximagesink->cur_image) {
-    gst_buffer_unref (ximagesink->cur_image);
+    gst_buffer_unref (GST_BUFFER_CAST (ximagesink->cur_image));
     ximagesink->cur_image = NULL;
   }
 
