@@ -154,7 +154,7 @@ struct _test_disc
 
 /* FIXME: now we just need to find out how to treat
  * data tracks for the cddb id calculation .... */
-struct _test_disc test_discs[NUM_TEST_DISCS] = {
+static struct _test_disc test_discs[NUM_TEST_DISCS] = {
   {nh_cd2_tracks, G_N_ELEMENTS (nh_cd2_tracks), 0xae11900e,
       NULL},
   {mb_sample_tracks, G_N_ELEMENTS (mb_sample_tracks), 0x00000000,
@@ -270,39 +270,78 @@ gst_cd_foo_src_read_sector (GstCddaBaseSrc * cddabasesrc, gint sector)
   return buf;
 }
 
-static gboolean
-plugin_init (GstPlugin * plugin)
+static inline gboolean
+tag_list_has_tag (GstTagList * list, const gchar * tag, GType type)
 {
-  if (!gst_element_register (plugin, "cdfoosrc", GST_RANK_SECONDARY,
-          GST_TYPE_CD_FOO_SRC))
+  const GValue *val = gst_tag_list_get_value_index (list, tag, 0);
+
+  if (val == NULL) {
+    GST_LOG ("no tag '%s' in taglist %" GST_PTR_FORMAT, tag, list);
     return FALSE;
+  }
+
+  if (!G_VALUE_HOLDS (val, type)) {
+    GST_LOG ("tag '%s' in taglist %" GST_PTR_FORMAT " is not of type %s",
+        tag, list, g_type_name (type));
+    return FALSE;
+  }
 
   return TRUE;
 }
 
-
-GST_PLUGIN_DEFINE_STATIC (GST_VERSION_MAJOR,
-    GST_VERSION_MINOR,
-    "cdfoosrc",
-    "Read audio from CD",
-    plugin_init, VERSION, "LGPL", GST_PACKAGE_NAME, GST_PACKAGE_ORIGIN);
-
 GST_START_TEST (test_discid_calculations)
 {
-  GstElement *foosrc;
+  GstElement *foosrc, *pipeline, *sink;
   gint i;
 
+  fail_unless (gst_element_register (NULL, "cdfoosrc", GST_RANK_SECONDARY,
+          GST_TYPE_CD_FOO_SRC));
+
+  pipeline = gst_pipeline_new ("pipeline");
+
+  sink = gst_element_factory_make ("fakesink", "sink");
+  fail_unless (sink != NULL, "couldn't create fakesink");
+
   foosrc = gst_element_factory_make ("cdfoosrc", "cdfoosrc");
+  fail_unless (foosrc != NULL, "couldn't create cdfoosrc");
+
+  gst_bin_add (GST_BIN (pipeline), foosrc);
+  gst_bin_add (GST_BIN (pipeline), sink);
+  fail_unless (gst_element_link (foosrc, sink));
 
   for (i = 0; i < G_N_ELEMENTS (test_discs); ++i) {
+    GstTagList *tags = NULL;
+    GstMessage *msg;
+
     GST_LOG ("Testing disc layout %u ...", i);
     GST_CD_FOO_SRC (foosrc)->cur_disc = i;
-    gst_element_set_state (foosrc, GST_STATE_PLAYING);
-    gst_element_get_state (foosrc, NULL, NULL, -1);
-    gst_element_set_state (foosrc, GST_STATE_NULL);
+    gst_element_set_state (pipeline, GST_STATE_PLAYING);
+
+    msg =
+        gst_bus_timed_pop_filtered (GST_ELEMENT_BUS (pipeline),
+        GST_CLOCK_TIME_NONE, GST_MESSAGE_TAG);
+    gst_message_parse_tag (msg, &tags);
+    fail_unless (tags != NULL);
+    fail_unless (tag_list_has_tag (tags, "track-count", G_TYPE_UINT));
+    fail_unless (tag_list_has_tag (tags, "track-number", G_TYPE_UINT));
+    fail_unless (tag_list_has_tag (tags, "duration", G_TYPE_UINT64));
+    fail_unless (tag_list_has_tag (tags, "discid", G_TYPE_STRING));
+    fail_unless (tag_list_has_tag (tags, "discid-full", G_TYPE_STRING));
+    fail_unless (tag_list_has_tag (tags, "musicbrainz-discid", G_TYPE_STRING));
+    fail_unless (tag_list_has_tag (tags, "musicbrainz-discid-full",
+            G_TYPE_STRING));
+    gst_tag_list_free (tags);
+    gst_message_unref (msg);
+
+    msg =
+        gst_bus_timed_pop_filtered (GST_ELEMENT_BUS (pipeline),
+        GST_CLOCK_TIME_NONE, GST_MESSAGE_ASYNC_DONE);
+    gst_message_unref (msg);
+
+    gst_element_set_state (pipeline, GST_STATE_NULL);
   }
 
-  gst_object_unref (foosrc);
+  gst_object_unref (pipeline);
 
   gst_task_cleanup_all ();
 }
@@ -316,12 +355,15 @@ GST_START_TEST (test_buffer_timestamps)
   GstPad *sinkpad;
   gint i;
 
+  fail_unless (gst_element_register (NULL, "cdfoosrc", GST_RANK_SECONDARY,
+          GST_TYPE_CD_FOO_SRC));
+
   pipeline = gst_pipeline_new ("pipeline");
   foosrc = gst_element_factory_make ("cdfoosrc", "cdfoosrc");
   fakesink = gst_element_factory_make ("fakesink", "fakesink");
   gst_bin_add_many (GST_BIN (pipeline), foosrc, fakesink, NULL);
   fail_unless (gst_element_link (foosrc, fakesink));
-  sinkpad = gst_element_get_pad (fakesink, "sink");
+  sinkpad = gst_element_get_static_pad (fakesink, "sink");
 
   GST_CD_FOO_SRC (foosrc)->cur_disc = 0;
 
@@ -372,19 +414,4 @@ cddabasesrc_suite (void)
   return s;
 }
 
-int
-main (int argc, char **argv)
-{
-  int nf;
-
-  Suite *s = cddabasesrc_suite ();
-  SRunner *sr = srunner_create (s);
-
-  gst_check_init (&argc, &argv);
-
-  srunner_run_all (sr, CK_NORMAL);
-  nf = srunner_ntests_failed (sr);
-  srunner_free (sr);
-
-  return nf;
-}
+GST_CHECK_MAIN (cddabasesrc)
